﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Services.Optimization.ControllerService.Interfaces;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.StorageClient;

namespace HeuristicLab.Services.Optimization.ControllerService.Mockup {
  public static class AzureConstants {
    public static readonly string SCENARIO_TABLE = "Scenario";
    public static readonly string SCENARIO_BLOB_CONTAINER = "scenario";
    public static readonly string CLOUD_SETTINGS_KEY = "Cloudia.WindowsAzure.Storage";
  }

  public class ScenarioDao : IScenarioDao {
    private static List<ScenarioEntity> scenarios;

    static ScenarioDao() {
      // TODO: Add tsp scenario to list
      scenarios = new List<ScenarioEntity>() {
        new ScenarioEntity("Benchmark Algorithm", "bmx", "bmm"),
        new ScenarioEntity("Traveling Salesman Problem", "scenario1", "mapper1")
      };      
    }


    public bool Add(ScenarioEntity entity) {
      var dbEntity = scenarios.Find(e => e.Scenario == entity.Scenario);

      if (dbEntity != null)
        return false;

      scenarios.Add(entity);
      return true;
    }

    public bool DeleteByName(string scenarioName) {
      return scenarios.RemoveAll(e => e.RowKey == scenarioName) == 1;
    }

    public ScenarioEntity FindByName(string scenarioName) {
      return scenarios.Find(e => e.RowKey == scenarioName);      
    }


    public IEnumerable<ScenarioEntity> GetAllEntities() {
      return new List<ScenarioEntity>(scenarios);
    }
  }

  public class BlobDao : IBlobDao {
    private static List<StringEntry> entries;

    static BlobDao() {
      entries = new List<StringEntry>() {
        new StringEntry() {
          Key="bmm",
          Text=@"
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Data;
using HeuristicLab.Services.Optimization.ControllerService.Model;
using HeuristicLab.Optimization;
using HeuristicLab.Services.Optimization.ControllerService;
using HeuristicLab.Algorithms.Benchmarks;
using HeuristicLab.Parameters;
using HeuristicLab.PluginInfrastructure;


namespace HeuristicLab.Mappers {
  public class BenchmarkScenarioMapper : IScenarioMapper {
    public void MapScenario(OptimizationScenario scenario, out IAlgorithm algorithm) {
      var alg = scenario.FirstAlgorithm;
      var bm = new BenchmarkAlgorithm();
      var bmFullName = HLMapper.GetStringValue(alg.Parameters.FindByName(""Benchmark""));
      bm.BenchmarkParameter.Value = bm.Benchmark = bm.BenchmarkParameter.ValidValues.First(b => b.GetType().FullName == bmFullName);      
      ((ValueParameter<IntValue>)bm.Parameters[""ChunkSize""]).Value = HLMapper.ConvertToIntValue(alg.Parameters.FindByName(""ChunkSize""));
      ((ValueParameter<DoubleValue>)bm.Parameters[""TimeLimit""]).Value = HLMapper.ConvertToDoubleValue(alg.Parameters.FindByName(""TimeLimit""));

      algorithm = bm;
    }
  }
}
"
        },
        new StringEntry() {
           Key="bmx",
           Text=@"<?xml version='1.0'?>
<scenario xmlns=""urn:scenario-schema"">
  <name>Benchmark Algorithm</name>
  <algorithm mapper=""HeuristicLab.Mappers.BenchmarkScenarioMapper"">
    <parameters>
		<param type=""Type"" name=""Benchmark"">
		  <choice name=""HeuristicLab.Algorithms.Benchmarks.Dhrystone"" selected=""true"" />
		  <choice name=""HeuristicLab.Algorithms.Benchmarks.Linpack"" />
		  <choice name=""HeuristicLab.Algorithms.Benchmarks.Whetstone""/>
		</param>    
		<param type=""Decimal"" name=""ChunkSize"" value=""0""/>
		<param type=""Decimal"" name=""TimeLimit"" value=""0""/> 
	</parameters>
  </algorithm>  
</scenario>"
        },
        new StringEntry() { 
          Key = "scenario1",
          Text = @"<?xml version='1.0'?>
<scenario xmlns=""urn:scenario-schema"">
  <name>Traveling Salesman Problem</name>
  <algorithm mapper=""HeuristicLab.Mappers.TSPScenarioMapper"">
    <parameters>
		<param type=""Type"" name=""CrossoverParameter"">
		  <choice name=""OrderCrossover2""/>
		  <choice name=""CosaCrossover"" selected=""true""/>
		</param>    
		<param type=""Integer"" name=""Elites"" value=""1""/>
		<param type=""Integer"" name=""MaximumGenerations"" value=""1000""/>
		<param type=""Percent"" name=""MutationProbability"" value=""5""/>
		<param type=""Type"" name=""Mutator"">
		  <choice name=""InversionManipulator"" selected=""true""/>
		  <choice name=""ScrambleManipulator""/>
		</param>
		<param type=""Integer"" name=""PopulationSize"" value=""100""/>
		<param type=""Integer"" name=""Seed"" value=""100""/>
		<param type=""Type"" name=""Selector"">
		  <choice name=""ProportionalSelector"" selected=""true""/>
		  <choice name=""BestSelector""/>
		</param>
		<param type=""Boolean"" name=""SetSeedRandomly"" value=""true""/>
    </parameters>
	<problem>
		<parameters>
			<param type=""Decimal"" name=""BestKnownQuality"" value=""6110.0""/>
			<param type=""DecimalVector"" name=""BestKnownSolution"">
			  <value v1=""0""/>
			  <value v1=""40""/>
			  <value v1=""38""/>
			  <value v1=""60""/>
			</param>
			<param type=""DecimalMatrix"" name=""Coordinates"">
			  <value v1=""334.5909"" v2=""161.7809""/>
			  <value v1=""503.5909"" v2=""172.7809""/>
			  <value v1=""400.5909"" v2=""300.7809""/>
			  <value v1=""700.5909"" v2=""700.7809""/>
			</param>
			<param type=""Type"" name=""EvaluatorParameter"">
			  <choice name=""HeuristicLab.Problems.TravelingSalesman.TSPRoundedEuclideanPathEvaluator"" />
			  <choice name=""HeuristicLab.Problems.TravelingSalesman.TSPGeoPathEvaluator"" selected=""true""/>
			  <choice name=""HeuristicLab.Problems.TravelingSalesman.TSPEuclideanPathEvaluator""/>
			</param>    
			<param type=""Boolean"" name=""UseDistanceMatrix"" value=""true""/> 
		</parameters>
	</problem>
  </algorithm>
</scenario>"
        },
        new StringEntry() { 
          Key = "mapper1",
          Text = @"using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Data;
using HeuristicLab.Services.Optimization.ControllerService.Model;
using HeuristicLab.Optimization;
using HeuristicLab.Services.Optimization.ControllerService;
using HeuristicLab.Algorithms.GeneticAlgorithm;
using HeuristicLab.Problems.TravelingSalesman;
namespace HeuristicLab.Mappers {
  public class TSPScenarioMapper : IScenarioMapper {
    public void MapScenario(OptimizationScenario scenario, out IAlgorithm algorithm) {
      Console.WriteLine(""Mapping scenario!"");
      var ga = new GeneticAlgorithm();
      var problem = new TravelingSalesmanProblem();
      ga.Problem = problem;
      algorithm = ga;

      problem.BestKnownQuality = HLMapper.ConvertToDoubleValue(scenario.FirstAlgorithm.Problem.Parameters.FindByName(""BestKnownQuality""));
      problem.BestKnownSolution = HLMapper.ConvertToPermutation(scenario.FirstAlgorithm.Problem.Parameters.FindByName(""BestKnownSolution""));
      problem.Coordinates = HLMapper.ConvertToDoubleMatrix(scenario.FirstAlgorithm.Problem.Parameters.FindByName(""Coordinates""));
      var evalParam = HLMapper.GetStringValue(scenario.FirstAlgorithm.Problem.Parameters.FindByName(""EvaluatorParameter""));
      if (evalParam == ""HeuristicLab.Problems.TravelingSalesman.TSPRoundedEuclideanPathEvaluator"") {
        problem.EvaluatorParameter.Value = new TSPRoundedEuclideanPathEvaluator();
      }
      else if (evalParam == ""HeuristicLab.Problems.TravelingSalesman.TSPGeoPathEvaluator"") {
        problem.EvaluatorParameter.Value = new TSPGeoPathEvaluator();
      }
      else {
        problem.EvaluatorParameter.Value = new TSPEuclideanPathEvaluator();
      }
      problem.UseDistanceMatrix = HLMapper.ConvertToBoolValue(scenario.FirstAlgorithm.Problem.Parameters.FindByName(""UseDistanceMatrix""));
      Console.WriteLine(""Mapping algorithm..."");
      ga.Mutator = HLMapper.FindInItemSet<HeuristicLab.Optimization.IManipulator>(ga.MutatorParameter.ValidValues, scenario.FirstAlgorithm.Parameters.FindByName(""Mutator""));
      ga.CrossoverParameter.Value = HLMapper.FindOperator<HeuristicLab.Optimization.ICrossover>(problem, scenario.FirstAlgorithm.Parameters.FindByName(""CrossoverParameter""));
      ga.Elites = HLMapper.ConvertToIntValue(scenario.FirstAlgorithm.Parameters.FindByName(""Elites""));
      ga.MaximumGenerations = HLMapper.ConvertToIntValue(scenario.FirstAlgorithm.Parameters.FindByName(""MaximumGenerations""));
      ga.MutationProbability = HLMapper.ConvertToPercentValue(scenario.FirstAlgorithm.Parameters.FindByName(""MutationProbability""));
      ga.PopulationSize = HLMapper.ConvertToIntValue(scenario.FirstAlgorithm.Parameters.FindByName(""PopulationSize""));
      ga.Seed = HLMapper.ConvertToIntValue(scenario.FirstAlgorithm.Parameters.FindByName(""Seed""));
      ga.Selector = HLMapper.FindInItemSet<HeuristicLab.Optimization.ISelector>(ga.SelectorParameter.ValidValues, scenario.FirstAlgorithm.Parameters.FindByName(""Selector""));
      ga.SetSeedRandomly = HLMapper.ConvertToBoolValue(scenario.FirstAlgorithm.Parameters.FindByName(""SetSeedRandomly""));
    }
  }
}"
        }
      };
    }

    public bool Add(StringEntry entry) {
      entries.RemoveAll(e => e.Key == entry.Key);
      entries.Add(entry);      
      return true;
    }

    public bool DeleteByKey(string entryKey) {
      return entries.RemoveAll(e => e.Key == entryKey) == 1;      
    }

    public StringEntry FindByKey(string entryKey) {
      return entries.Find(e => e.Key == entryKey);      
    }
  }

  public class MockupDataAccessLayer : IDataAccessLayer {
    private IScenarioDao scenarioDao;
    private IBlobDao blobDao;

    public IScenarioDao ScenarioDao {
      get {
        if (scenarioDao == null) {
          scenarioDao = new ScenarioDao();
        }
        return scenarioDao;
      }
    }

    public IBlobDao BlobDao {
      get {
        if (blobDao == null) {
          blobDao = new BlobDao();
        }
        return blobDao;
      }
    }

    private IExperimentDao experimentDao;

    public IExperimentDao ExperimentDao {
      get {         
        return null; 
      }      
    }
  }
}
